using System.Collections.Generic;
using Xunit;

namespace csharpcore
{
    public class BackstagePassesTest
    {
        private readonly GildedRose _rose;
        private List<Item> _items = new List<Item>();

        public BackstagePassesTest()
        {
            _rose = new GildedRose(_items);
        }

        [Fact]
        public void ShouldIncreaseQualityWhenSellInApproaches()
        {
            var item = GenerateItem(0, 30);
            
            _rose.UpdateQuality();
            Assert.Equal("Backstage passes to a TAFKAL80ETC concert", item.Name);
            Assert.Equal(29, item.SellIn);
            Assert.Equal(1, item.Quality);
            
            _rose.UpdateQuality();
            Assert.Equal("Backstage passes to a TAFKAL80ETC concert", item.Name);
            Assert.Equal(28, item.SellIn);
            Assert.Equal(2, item.Quality);
        }
        
        [Theory]
        [InlineData(10)]
        [InlineData(8)]
        public void ShouldIncreaseQualityByTwoWhenSellInIsInTenDaysOrLess(int sellInDays)
        {
            const int startQuality = 42;
            var item = GenerateItem(startQuality, sellInDays);
            _rose.UpdateQuality();

            Assert.Equal(startQuality + 2, item.Quality);
        }

        [Theory]
        [InlineData(5)]
        [InlineData(3)]
        public void ShouldIncreaseQualityByThreeWhenSellInIsInFiveDaysOrLess(int sellInDays)
        {
            const int startQuality = 42;
            var item = GenerateItem(startQuality, sellInDays);
            _rose.UpdateQuality();
            
            Assert.Equal(startQuality + 3, item.Quality);
        }

        [Theory]
        [InlineData(0)]
        [InlineData(-1)]
        public void ShouldDropQualityToZeroWhenSellInIsPassed(int sellIn)
        {
            var item = GenerateItem(20, sellIn);
            
            _rose.UpdateQuality();
            
            Assert.Equal("Backstage passes to a TAFKAL80ETC concert", item.Name);
            Assert.Equal(sellIn - 1, item.SellIn);
            Assert.Equal(0, item.Quality);
        }
        
        private Item GenerateItem(int q, int s)
        {
            var item = new Item {Name = "Backstage passes to a TAFKAL80ETC concert", Quality = q, SellIn = s};
            _items.Clear();
            _items.Add(item);
            return item;
        }
    }
}